import * as React from 'react';
import {observer} from "mobx-react-lite";
import {DashboardComponentModel} from "../../../stores/DashboardComponentStore";
import {DashboardTargetModel} from "../../../stores/DashboardTargetStore";
import {CheckboxField, InputField, SelectField, useReadonlyContext} from "@rx-suite/ui";
import {Parameter} from "../../../stores/Types";
import {t} from "@lingui/macro";
import {ComplexParameterField} from "./ComplexParameterField";

export type ParameterFieldProps = {
    component: DashboardComponentModel,
    parameter: Parameter
}

export const ParameterField: React.FunctionComponent<ParameterFieldProps> = observer(({component, parameter}) => {
    const target = component.viewModel.dashboard_target as DashboardTargetModel;
    const readOnly = useReadonlyContext();
    const value = component.viewModel.parameters[parameter.id];
    const name = `parameters.${parameter.id}`;
    const label = parameter.name;
    if (!target) {
        return null;
    }

    const hint = {
        title: parameter.name,
        children: <>
            {parameter.description}
            {(parameter.max || (parameter.max === 0)) &&
            <div>{t`Maximum Value:`}{parameter.max}</div>}
            {(parameter.min || (parameter.min === 0)) &&
            <div>{t`Minimum Value:`}{parameter.min}</div>}
            {(parameter.required) && <div>{t`This parameter is required`}</div>}
            {(parameter.value_list) && <div>{t`Available Values:`}
                <ul>
                    {parameter.value_list.map(v => <li key={v.value}>
                        {v.label}{v.description ? `: ${v.description}` : ''}
                    </li>)}
                </ul>
            </div>}</>
    }

    switch (parameter.type) {
        case 'integer':
            const inputProps = {} as { min?: number, max?: number };
            if (parameter.min || (parameter.min === 0)) {
                inputProps.min = parameter.min;
            }
            if (parameter.max || (parameter.max === 0)) {
                inputProps.max = parameter.max;
            }
            return <InputField label={label}
                               name={name}
                               value={(value || value === 0) ? value : ''}
                               disabled={readOnly}
                               type={'number'}
                               inputProps={inputProps}
                               hint={hint}
                               onChange={component.handleChange}/>
        case 'boolean':
            return <CheckboxField label={label}
                                  name={name}
                                  checked={!!value}
                                  disabled={readOnly}
                                  hint={hint}
                                  onChange={component.handleChange}/>
        case 'string':
            if (parameter.value_list) {
                return <SelectField onChange={component.handleSelectChange}
                                    label={label}
                                    multiple={parameter.multiple}
                                    onCreateOption={() => {
                                    }}
                                    hint={hint}
                                    options={parameter.value_list.reduce((options: { label: string, options: { label: string, value: string }[] }[], valueDef) => {
                                        let groupName = valueDef.group || '';
                                        let group = options.find((o: { label: string }) => o.label === groupName);
                                        if (!group) {
                                            group = {
                                                label: groupName,
                                                options: []
                                            };
                                            options.push(group);
                                        }
                                        group.options.push({label: valueDef.label as string, value: valueDef.value});
                                        return options
                                    }, [])}
                                    name={name}
                                    disabled={readOnly}
                                    value={value || (parameter.multiple ? [] : '')}/>
            } else {
                return <InputField label={label}
                                   name={name}
                                   value={(value || value === 0) ? value : ''}
                                   disabled={readOnly}
                                   hint={hint}
                                   onChange={component.handleChange}/>
            }
        default:
            return <ComplexParameterField parameter={parameter} component={component}/>
    }
});